@@ -69,7 +69,7 @@ _verbosity_sticky = 1
 _verbosity = 1
 _output_mode_sticky = _OM_TEXT
 _output_mode = _OM_TEXT
-_ctxt_settings = dict(help_option_names=['-?','-h','--help'],ignore_unknown_options=True,allow_extra_args=True)
+_ctxt_settings = dict(help_option_names=['-h','--help'],ignore_unknown_options=True,allow_extra_args=True)
 _task_statuses = {0: "Queued", 1: "Ready", 2: "Running", 3: "Succeeded", 4: "Failed"}
 _task_types = { 0: "Data Get", 1: "Data Put", 2: "Data Del", 3: "Rec Chg Alloc", 4: "Rec Chg Owner", 5: "Rec Delete", 6: "Alloc Create", 7: "Alloc Del", 8: "User Del", 9: "Project Del"}
 
@@ -77,6 +77,7 @@ _task_types = { 0: "Data Get", 1: "Data Put", 2: "Data Del", 3: "Rec Chg Alloc",
 _initialized = False
 _devnull = None
 
+_hdr_lev_char = ['-','-','^',',']
 
 # =============================================================================
 # --------------------------------------- CLI Module Public Interface Functions
@@ -399,12 +400,14 @@ def _global_output_options(func):
 @click.option("--version",is_flag=True,help="Print version number and exit.")
 @click.pass_context
 def _cli(ctx,*args,**kwargs):
-    ''''datafed' is the command-line interface (_cli) for the DataFed federated data management
-    service. This _cli may be used to access most, but not all, of the features available
-    via the DataFed web portal. This _cli may be used interactively (-i option), or for
-    scripting (supports JSON output with the -J option).
+    ''''datafed' is the command-line interface (CLI) for the DataFed federated data management
+    service and may be used to access many of the features available via the DataFed web
+    portal. This CLI may be used interactively (human-friendly output) or for scripting (JSON
+    output) by specifying the -s option.
 
-    For more information about this _cli and DataFed in general, refer to https://datafed.ornl.gov/ui/docs
+    When the datafed CLI is run without any command arguments, a interactive shell session is
+    started. While in the shell, commands should be entered without specifying the 'datafed'
+    prefix.
     '''
 
     global _verbosity
@@ -424,51 +427,90 @@ def _cli(ctx,*args,**kwargs):
 @_cli.command(name='gendoc',hidden=True)
 @click.pass_context
 def _genDoc( ctx ):
-    toc = []
 
-    #print( _cli.get_help( ctx.parent ), "\n" )
+    body = _genDocCmd( None, ctx, 0, recurse = False )
 
-    body = "<h1 id='main'>Main</h1>\n\n<pre>" + _cli.get_help( ctx.parent ) + "</pre>\n"
-    toc.append("<a href='#main'>Main</a>")
+    #body = _genDocHeader("General Usage",0) + "\n" + _cli.get_help( ctx.parent ) + "\n\n"
 
-    sec = 1
     for c in _cli.list_commands( ctx ):
         subcmd = _cli.get_command( _cli, c )
         if not subcmd.hidden:
-            body = body + _genDocCmd( subcmd, click.Context( subcmd, info_name = subcmd.name, parent=ctx.parent ), str(sec), "", 2, toc )
-            sec = sec + 1
+            body = body + _genDocCmd( subcmd, click.Context( subcmd, info_name = subcmd.name, parent=ctx.parent ), 0 )
 
-    _toc = ""
-    for t in toc:
-        _toc = _toc + t + "<br>\n"
+    print(body)
 
-    print("<html><head><title>DataFed CLI Help</title></head><body style='margin:0;padding:0'><div style='display:flex;flex-direction:column;height:100%;width:100%'><div style='flex:none;background:#4040bb;color:#ffffff;padding:.5em'><span style='font-size:2em'>DataFed Command Line Interface Help</span>&nbsp&nbsp&nbsp&nbspCLI V-{}<br>This documentation is automatically generated from the latest released DataFed CLI, available as the 'datafed' package on PyPi.</div><div style='flex:1 1 auto;display:flex;flex-direction:row;min-height:0'><div style='flex:none;overflow:auto;padding:.25em;background:#bbbbbb'>{}</div><div style='flex:1 1 auto;overflow:auto;padding: 0em 2em 0em 2em'>{}</div></div></div></body></html>".format( version, _toc, body ))
+def _genDocHeader( cmd, level ):
+    global _hdr_lev_char
+    ul = ""
+    ul = ul.rjust( len(cmd), _hdr_lev_char[level])
 
+    if level == 0:
+        return ul + "\n" + cmd + "\n" + ul + "\n"
+    else:
+        return cmd + "\n" + ul + "\n"
+
+def _genDocCmd( cmd, ctx, level, parname = None, recurse = True ):
+    if cmd == None:
+        cname = "Datafed"
+        cmd = _cli
+    elif parname:
+        cname = parname + " " + cmd.name.capitalize()
+    else:
+        cname = cmd.name.capitalize()
 
-def _genDocCmd( cmd, ctx, section, path, hlev, toc ):
     if hasattr( cmd, 'list_commands' ):
         is_group = True
+        doc = _genDocHeader( cname + " Commands", level )
     else:
         is_group = False
+        doc = _genDocHeader( cname + " Command", level )
 
-    cname = cmd.name.capitalize()
-    html = "\n<h{} id='s{}'>{} {}{}{}</h{}>\n\n<pre>".format(hlev,section,section,path,cname," Commands" if is_group else "",hlev) + cmd.get_help( ctx ) + "</pre>\n"
-    toc.append("<a href='#s{}'>{} {}{}{}</a>".format(section,section,path,cname," Commands" if is_group else ""))
+    #doc += "\n" + cmd.get_help( ctx ) + "\n\n"
+
+    tmp = cmd.collect_usage_pieces( ctx )
+    opts = []
+    for param in cmd.get_params(ctx):
+        rv = param.get_help_record(ctx)
+        if rv is not None:
+            opts.append(rv)
+
+    doc +=  "\n" + cmd.help + "\n\nUsage::\n\n    " 
+    if cname == "Datafed":
+        doc += "datafed"
+    else:
+        doc += ctx.command_path 
+    doc += " " + " ".join(tmp) + "\n\nOptions:\n\n"
+
+    for o in opts:
+        doc += o[0] + "  " + o[1] + "\n"
+
+    doc += "\n"
 
     if is_group:
-        sec = 1
+        doc += "Sub-Commands:\n\n===============  ============================================================\n"
         for c in cmd.list_commands( ctx ):
             subcmd = cmd.get_command( cmd, c )
             if not subcmd.hidden:
-                html = html + _genDocCmd( subcmd, click.Context( subcmd, info_name = subcmd.name, parent=ctx ), section + "." + str(sec), path + cname + " ", hlev + 1, toc )
-                sec = sec + 1
+                doc += '{0: <15}  {1}\n'.format( subcmd.name, subcmd.get_short_help_str(limit=60))
+        doc += "===============  ============================================================\n\n"
 
-    return html
+        if recurse:
+            for c in cmd.list_commands( ctx ):
+                subcmd = cmd.get_command( cmd, c )
+                if not subcmd.hidden:
+                    doc = doc + _genDocCmd( subcmd, click.Context( subcmd, info_name = subcmd.name, parent=ctx ), level + 1, cname )
+    else:
+        doc += "\n"
+
+    return doc
 
 @_cli.command(name='wc')
 @click.argument("coll_id",required=False, metavar="ID")
 def _wc( coll_id ):
-    '''Set/print current working collection or path. 'ID' can be a collection ID, alias, list index number, '-' (previous collection), or path. Only '..' and '/' are supported for paths. 'cd' is an alias for this command.
+    '''
+    Set/print current working collection or path. 'ID' can be a collection ID, alias,
+    list index number, '-' (previous collection), or path. Only '..' and '/' are
+    supported for paths. 'cd' is an alias for this command.
     '''
 
     if _output_mode_sticky != _OM_RETN and not _interactive:
@@ -579,7 +621,7 @@ def _alias( context ):
 # =============================================================================
 
 
-@_cli.command(name='data',cls=_AliasedGroup,help="Data subcommands.")
+@_cli.command(name='data',cls=_AliasedGroup,help="Data commands.")
 def _data():
     pass
 
@@ -591,7 +633,7 @@ def _data():
 @click.argument( "data_id", metavar="ID" )
 def _dataView( data_id, context ):
     '''
-    View data record information. Displays record title, description, keywords,
+    View data record information. Displays record title, description, tags,
     and other informational and administrative fields. ID may be a data record
     identifier, alias, or index value from a listing. By default, description
     text is truncated and metadata is not shown unless the verbosity is as
@@ -605,7 +647,7 @@ def _dataView( data_id, context ):
 @click.argument("title", required=True)
 @click.option("-a","--alias",type=str,required=False,help="Record alias.")
 @click.option("-d","--description",type=str,required=False,help="Description text.")
-@click.option("-k","--keywords",type=str,required=False,help="Keywords (comma separated list).")
+@click.option("-T","--tags",type=str,required=False,help="Tags (comma separated list).")
 @click.option("-r","--raw-data-file",type=str,required=False,help="Globus path to raw data file (local or remote) to upload to new record. Default endpoint is used if none provided.")
 @click.option("-x","--extension",type=str,required=False,help="Override raw data file extension if provided (default is auto detect).")
 @click.option("-m","--metadata",type=str,required=False,help="Inline metadata in JSON format. JSON must define an object type. Cannot be specified with --metadata-file option.")
@@ -615,7 +657,7 @@ def _dataView( data_id, context ):
 @click.option("-D","--deps",multiple=True, type=click.Tuple([click.Choice(['der', 'comp', 'ver']), str]),help="Dependencies (provenance). Use one '--deps' option per dependency and specify with a string consisting of the type of relationship ('der', 'comp', 'ver') follwed by ID/alias of the referenced record. Relationship types are: 'der' for 'derived from', 'comp' for 'a component of', and 'ver' for 'a new version of'.")
 @_global_context_options
 @_global_output_options
-def _dataCreate( title, alias, description, keywords, raw_data_file, extension, metadata, metadata_file, parent, repository, deps, context ):
+def _dataCreate( title, alias, description, tags, raw_data_file, extension, metadata, metadata_file, parent, repository, deps, context ):
     '''
     Create a new data record. The data record 'title' is required, but all
     other attributes are optional. On success, the ID of the created data
@@ -632,12 +674,15 @@ def _dataCreate( title, alias, description, keywords, raw_data_file, extension,
     if metadata and metadata_file:
         raise Exception( "Cannot specify both --metadata and --metadata-file options." )
 
+    if tags:
+        tags = tags.split(",")
+
     if parent:
         parent_id = _resolve_coll_id( parent, context )
     else:
         parent_id = _cur_coll
 
-    reply = _capi.dataCreate( title, alias = alias, description = description, keywords = keywords, extension = extension,
+    reply = _capi.dataCreate( title, alias = alias, description = description, tags = tags, extension = extension,
         metadata = metadata, metadata_file = metadata_file, parent_id = parent_id, deps = deps, repo_id = repository, context = context )
     _generic_reply_handler( reply, _print_data )
 
@@ -652,7 +697,7 @@ def _dataCreate( title, alias, description, keywords, raw_data_file, extension,
 @click.option("-t","--title",type=str,required=False,help="Title")
 @click.option("-a","--alias",type=str,required=False,help="Alias")
 @click.option("-d","--description",type=str,required=False,help="Description text")
-@click.option("-k","--keywords",type=str,required=False,help="Keywords (comma separated list)")
+@click.option("-T","--tags",type=str,required=False,help="Tags (comma separated list)")
 @click.option("-r","--raw-data-file",type=str,required=False,help="Globus path to raw data file (local or remote) to upload with record. Default endpoint used if none provided.")
 @click.option("-x","--extension",type=str,required=False,help="Override extension for raw data file (default = auto detect).")
 @click.option("-m","--metadata",type=str,required=False,help="Inline metadata in JSON format.")
@@ -662,7 +707,7 @@ def _dataCreate( title, alias, description, keywords, raw_data_file, extension,
 @click.option("-R","--deps-rem",multiple=True, nargs=2, type=click.Tuple([click.Choice(['der', 'comp', 'ver']), str]),help="Specify dependencies to remove by listing first the type of relationship ('der', 'comp', or 'ver') followed by ID/alias of the target record. Can be specified multiple times.")
 @_global_context_options
 @_global_output_options
-def _dataUpdate( data_id, title, alias, description, keywords, raw_data_file, extension, metadata, metadata_file, metadata_set, deps_add, deps_rem, context ):
+def _dataUpdate( data_id, title, alias, description, tags, raw_data_file, extension, metadata, metadata_file, metadata_set, deps_add, deps_rem, context ):
     '''
     Update an existing data record. The data record ID is required and can be
     an ID, alias, or listing index; all other record attributes are optional.
@@ -676,7 +721,10 @@ def _dataUpdate( data_id, title, alias, description, keywords, raw_data_file, ex
     if metadata and metadata_file:
         raise Exception( "Cannot specify both --metadata and --metadata-file options." )
 
-    reply = _capi.dataUpdate( _resolve_id( data_id ), title = title, alias = alias, description = description, keywords = keywords, extension = extension,
+    if tags:
+        tags = tags.split(",")
+
+    reply = _capi.dataUpdate( _resolve_id( data_id ), title = title, alias = alias, description = description, tags = tags, extension = extension,
         metadata = metadata, metadata_file = metadata_file, metadata_set = metadata_set, deps_add = deps_add, deps_rem = deps_rem, context = context )
     _generic_reply_handler( reply, _print_data )
 
@@ -751,7 +799,7 @@ def _dataGet( df_id, path, wait, encrypt, orig_fname, context ):
 
     reply = _capi.dataGet( resolved_ids, path, encrypt = int(encrypt), orig_fname = orig_fname, wait = wait, progress_bar = bar, context = context )
 
-    if reply[1] == "DataGetPutReply":
+    if reply[1] == "DataGetReply":
         _generic_reply_handler( reply, _print_task )
 
 
@@ -774,7 +822,7 @@ def _dataPut( data_id, path, wait, extension, encrypt, context ):
 
     reply = _capi.dataPut( _resolve_id( data_id ), path, encrypt = int(encrypt), wait = wait, extension = extension, context = context )
 
-    if reply[1] == "DataGetPutReply":
+    if reply[1] == "DataPutReply":
         _generic_reply_handler( reply, _print_task )
     else:
         _generic_reply_handler( reply, _print_task_array )
@@ -784,7 +832,7 @@ def _dataPut( data_id, path, wait, extension, encrypt, context ):
 # -------------------------------------------------------- Data-Batch Functions
 # =============================================================================
 
-@_data.command(name='batch',cls=_AliasedGroup,help="Data batch subcommands.")
+@_data.command(name='batch',cls=_AliasedGroup,help="Data batch commands.")
 def _batch():
     pass
 
@@ -835,9 +883,11 @@ def _data_batch_update( file ):
 @click.pass_context
 def _list( ctx, item_id, offset, count, context ):
     '''
-    List contents of a collection, or shared items of a user or project.
-    ID may be a collection ID or alias, a user or project ID, an index
-    value from a listing, or omitted for the current working collection.
+    List contents of a collection, or shared items. ID may be a collection ID
+    or alias, a relative path, a user or project ID, an index value from a
+    listing, or omitted for the current working collection. If the ID is a
+    user or project, the ls command will list shared items associated with the
+    given user or project.
     '''
 
     global _cur_coll
@@ -870,7 +920,7 @@ def _list( ctx, item_id, offset, count, context ):
 # =============================================================================
 
 
-@_cli.command( name='coll',cls=_AliasedGroup, help="Collection subcommands." )
+@_cli.command( name='coll',cls=_AliasedGroup, help="Collection commands." )
 def _coll():
     pass
 
@@ -896,10 +946,11 @@ def _collView( coll_id, context ):
 @click.option("-a","--alias",type=str,required=False,help="Alias")
 @click.option("-p","--parent",type=str,required=False,help="Parent collection ID/alias (default is current working collection)")
 @click.option("-d","--description",type=str,required=False,help="Description text")
+@click.option("-T","--tags",type=str,required=False,help="Tags (comma separated list).")
 @click.option("--topic", type=str, required=False, help="Publish the collection to the provided topic.")
 @_global_context_options
 @_global_output_options
-def _collCreate( title, alias, description, topic, parent, context ):
+def _collCreate( title, alias, description, tags, topic, parent, context ):
     '''
     Create a new collection. The collection 'title' is required, but all
     other attributes are optional. On success, the ID of the created
@@ -913,7 +964,10 @@ def _collCreate( title, alias, description, topic, parent, context ):
     else:
         parent_id = _cur_coll
 
-    reply = _capi.collectionCreate( title, alias = alias, description = description, topic = topic, parent_id = parent_id, context = context )
+    if tags:
+        tags = tags.split(",")
+
+    reply = _capi.collectionCreate( title, alias = alias, description = description, tags = tags, topic = topic, parent_id = parent_id, context = context )
     _generic_reply_handler( reply, _print_coll )
 
 
@@ -922,17 +976,21 @@ def _collCreate( title, alias, description, topic, parent, context ):
 @click.option("-t","--title",type=str,required=False,help="Title")
 @click.option("-a","--alias",type=str,required=False,help="Alias")
 @click.option("-d","--description",type=str,required=False,help="Description text")
+@click.option("-T","--tags",type=str,required=False,help="Tags (comma separated list).")
 @click.option("--topic", type=str, required=False, help="Publish the collection under the provided topic.")
 @_global_context_options
 @_global_output_options
-def _collUpdate( coll_id, title, alias, description, topic, context):
+def _collUpdate( coll_id, title, alias, description, tags, topic, context):
     '''
     Update an existing collection. The collection ID is required and can be
     an ID, alias, or listing index; all other collection attributes are
     optional.
     '''
 
-    reply = _capi.collectionUpdate( _resolve_coll_id( coll_id, context = context ), title = title, alias = alias, description = description, topic = topic, context = context )
+    if tags:
+        tags = tags.split(",")
+
+    reply = _capi.collectionUpdate( _resolve_coll_id( coll_id, context = context ), title = title, alias = alias, description = description, tags = tags, topic = topic, context = context )
     _generic_reply_handler( reply, _print_coll )
 
 
@@ -1014,7 +1072,7 @@ def _coll_rem(  coll_id, item_id, context ):
 # ------------------------------------------------------------- Query Functions
 # =============================================================================
 
-@_cli.command(name='query',cls=_AliasedGroup,help="Data query subcommands.")
+@_cli.command(name='query',cls=_AliasedGroup,help="Data query commands.")
 def _query(*args,**kwargs):
     pass
 
@@ -1122,7 +1180,7 @@ def _queryRun( id, text, meta, no_default, coll, proj, offset, count ):
 # -------------------------------------------------------------- User Functions
 # =============================================================================
 
-@_cli.command(name='user',cls=_AliasedGroup,help="User subcommands.")
+@_cli.command(name='user',cls=_AliasedGroup,help="User commands.")
 def _user():
     pass
 
@@ -1185,7 +1243,7 @@ def _userWho():
 # ----------------------------------------------------------- Project Functions
 # =============================================================================
 
-@_cli.command(name='project',cls=_AliasedGroup,help="Project subcommands.")
+@_cli.command(name='project',cls=_AliasedGroup,help="Project commands.")
 def _project():
     pass
 
@@ -1513,8 +1571,8 @@ def _verbositySet(level):
 @click.pass_context
 def _help_cli(ctx,command):
     '''
-    Show DataFed CLI help. Include a subcommand name as the argument to see
-    subcommand-specific help.
+    Show DataFed CLI help. Include a command name as the argument to see
+    command-specific help.
     '''
 
     if not command:
@@ -1643,7 +1701,7 @@ def _print_user_listing( message ):
     _list_items = []
     for i in message.user:
         _list_items.append(i.uid)
-        click.echo("{:2}. {:24} {}".format(df_idx,i.uid,i.name))
+        click.echo("{:2}. {:24} {}, {}".format(df_idx,i.uid,i.name_last,i.name_first))
         df_idx += 1
 
 
@@ -1689,7 +1747,7 @@ def _print_data( message ):
         click.echo( "{:<15}{:<50}".format('ID: ', dr.id))
         click.echo( "{:<15}{:<50}".format('Alias: ', dr.alias if dr.alias else "(none)" ))
         _wrap_text( dr.title, "Title:", 15 )
-        _wrap_text( dr.keyw, "Keywords:", 15 )
+        _wrap_text( _arrayToCSV( dr.tags, 0 ), "Tags:", 15 )
 
         if dr.data_url:
             click.echo("{:<15}{:<50}".format('DOI No.: ', dr.doi))
@@ -1746,6 +1804,7 @@ def _print_coll( message ):
         click.echo( "{:<15}{:<50}".format('ID: ', coll.id))
         click.echo( "{:<15}{:<50}".format('Alias: ', coll.alias if coll.alias else "(none)" ))
         _wrap_text( coll.title, "Title:", 15 )
+        _wrap_text( _arrayToCSV( coll.tags, 0 ), "Tags:", 15 )
 
         click.echo( "{:<15}{:<50}".format('Topic: ', coll.topic if coll.topic else '(not published)') + '\n' +
                     "{:<15}{:<50}".format('Owner: ', coll.owner[2:]) + '\n' +
@@ -1800,10 +1859,10 @@ def _print_task( message ):
                     "{:<20} {:<50}".format('Updated: ', _capi.timestampToStr(message.task.ut)))
 
 def _print_task_array( message ):
-    print("_print_task_array")
+    #print("_print_task_array")
 
     for t in message.task:
-        print(t)
+        #print(t)
 
         task_type = _task_types.get(t.type, "None")
         task_status = _task_statuses.get(t.status, "None")
@@ -1826,7 +1885,7 @@ def _print_user( message ):
     for usr in message.user:
         if _verbosity >= 0:
             click.echo("{:<10} {:<50}".format('User ID: ', usr.uid) + '\n' +
-                       "{:<10} {:<50}".format('Name: ', usr.name) + '\n' +
+                       "{:<10} {:<50}".format('Name: ', usr.name_last + ", " + usr.name_first) + '\n' +
                        "{:<10} {:<50}".format('Email: ', usr.email))
 
 def _print_proj( message ):
@@ -2202,6 +2261,9 @@ def _initialize( opts ):
 
         _capi = CommandLib.API( opts )
 
+        if _interactive and _capi._mapi.new_client_avail:
+            click.echo("Note: Your DataFed python package is out of date.")
+
         if man_auth or _capi.getAuthUser() == None:
             if not man_auth:
                 if not _capi._mapi.keysLoaded():
